#!/bin/sh
#
# landisk-update.sh
#
# $Id$
#

PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin

MODEL=`sed -n '/^model/s/^model	//p' /etc/landisk/conffiles/config-system`
VERSION=`sed -n '/^version/s/^version	//p' /etc/landisk/conffiles/config-system`

ROOTPATH='/mnt/hda5/.update'

VERFILE="${ROOTPATH}/for_${MODEL}_series"
UPFILE="/boot/.update/update.tgz"
TARFILE="${ROOTPATH}/update-files.tar.gz"
BETAFILE="${ROOTPATH}/beta"
UPPERL="${ROOTPATH}/update.pl"
CONFDIR='/mnt/hda5/landisk/conffiles'
CONFTAR="${ROOTPATH}/conf_${MODEL}.tgz"
RMLIST="${ROOTPATH}/rmlist"
FIXFILE="${ROOTPATH}/fix_112.tgz"
MOTDEF="${ROOTPATH}/r8firm.def"
MOTFILE=`sed -n 's/R8MOTFILE[[:blank:]]*//p' $MOTDEF`
MOTDEF2="${ROOTPATH}/r8firm_pwr.def"
MOTFILE2=`sed -n 's/R8MOTFILE[[:blank:]]*//p' $MOTDEF2`
THADDR='00:A0:B0:96:00:00'
THADDR_R2U='00:A0:B0:9A:C1:F6'

UPDATEVER=`sed 's/-.*$//' $VERFILE`
if [ "$MODEL" = "HDL-GW" -o "$MODEL" = "HDL-GZ" ]; then
    UPDATESUBVER=`sed 's/^[0-9]*\.[0-9]*//' $VERFILE`
fi
[ -f $BETAFILE ] && BETA=`cat $BETAFILE`

start_led() {
   /usr/local/bin/ledcont stsfastblink 
}

exit_select() {
#    /usr/local/bin/ledcont errfastblink
#    /usr/local/bin/buzcont 10 50 3
    exit $1
}


###############################################################################
# ディスクラベルの値を読み込む
###############################################################################
__disklabel_ddread() {
    local DEVICE="${1}"
    local OFFSET="${2}"
    local LENGTH="${3}"
    local OUTPUT="$(dd if=/dev/${DEVICE} bs=1 skip=${OFFSET} count=${LENGTH} 2>/dev/null | tr -d '\000')"

    if [ "x${OUTPUT}" = "x" ]; then
	echo
	return 1
    else
	echo "${OUTPUT}"
	return 0
    fi

    return 1
}


###############################################################################
# conffilesの更新
###############################################################################
__update_conffiles() {
	local f=''
	local verup_files='config-ups iomail.ups config-alert config-quota'

	# conffilesの追加
	for f in $verup_files
	do
		if [ ! -e $CONFDIR/$f ]; then
			tar xpzf ${CONFTAR} \
			-C /mnt/hda5 landisk/conffiles/$f ||return 1
		fi
	done

	# config-serviceにapcupsd追加
	if ! grep -q 'apcupsd' $CONFDIR/config-service; then
		cat <<EOF >> $CONFDIR/config-service
apcupsd
	switch	0
	seq	99
EOF
	fi

	# config-ups のTIMEOUT を変更
	CONFIG_UPS="$CONFDIR/config-ups"
	if [ -e $CONFIG_UPS ]; then
		TMP_UPS='/tmp/.config-ups'
		sed -e '/^config/,/^[a-zA-Z0-9]/{s/TIMEOUT	1000/TIMEOUT	600/}' $CONFIG_UPS > $TMP_UPS
		mv $TMP_UPS $CONFIG_UPS
		chown 0:0 $CONFIG_UPS
		chmod 666 $CONFIG_UPS
	fi

	# config-smbにconfig=追加
	if ! grep -q 'config=' $CONFDIR/config-smb; then
		cat <<EOF >> $CONFDIR/config-smb
config=
	audit	0
EOF
	fi

	# config-smbにmsdfsエントリとwinbind_enumエントリを追加する
	sed -e '/^global/,/^[a-zA-Z0-9]/{/host_msdfs/D}' \
	    -e '/^global/a\	host_msdfs	no' \
	    -e '/^global/,/^[a-zA-Z0-9]/{/msdfs_root/D}' \
	    -e '/^global/a\	msdfs_root	no' \
	    -e '/^global/,/^[a-zA-Z0-9]/{/winbind_enum_users/D}' \
	    -e '/^global/a\	winbind_enum_users	yes' \
	    -e '/^global/,/^[a-zA-Z0-9]/{/winbind_enum_groups/D}' \
	    -e '/^global/a\	winbind_enum_groups	yes' \
	    -e '/^global/,/^[a-zA-Z0-9]/{/lanman_auth/D}' \
	    -e '/^global/a\	lanman_auth	yes' \
	    -e '/^global/,/^[a-zA-Z0-9]/{/client_lanman_auth/D}' \
	    -e '/^global/a\	client_lanman_auth	yes' \
	    -e '/^global/,/^[a-zA-Z0-9]/{/client_plaintext_auth/D}' \
	    -e '/^global/a\	client_plaintext_auth	yes' \
	    -e '/^global/,/^[a-zA-Z0-9]/{/usershare_max_shares/D}' \
	    -e '/^global/a\	usershare_max_shares	0' \
	    ${CONFDIR}/config-smb > /tmp/.config-smb || return 1

	mv /tmp/.config-smb ${CONFDIR}/config-smb || return 1
	chown 0:0 ${CONFDIR}/config-smb
	chmod 666 ${CONFDIR}/config-smb

	if [ "x$runlevel" = "xS" ]; then
		# 初期化時にspindown設定を0にする
		if [ -e $CONFDIR/config-disk ]; then
			sed -e 's/\tspindown.*/\tspindown\t0/g' $CONFDIR/config-disk > /tmp/config-disk 2>/dev/null
			mv /tmp/config-disk $CONFDIR/config-disk
			chmod 666 $CONFDIR/config-disk
		fi

	fi

	# アクティブリペアを有効にする
	# sed用のテンポラリファイルを一時的に作成し、これをsedに渡す。
	CONFIG_CRONTAB=$CONFDIR/config-crontab
	if [ -e $CONFIG_CRONTAB ]; then
		if ! grep -q 'spot_repair' $CONFIG_CRONTAB; then
			TMP_SCRIPT=/tmp/tmp.config-crontab.script
			TMP_CRONTAB=/tmp/config-crontab

			rm -f $TMP_SCRIPT > /dev/null 2> /dev/null
			cat <<EOF > $TMP_SCRIPT
/^crontab/ a \\
	spot_repair\\
		hour	2\\
		minute	0\\
		month	*\\
		user	root\\
		dayofweek	0\\
		dayofmonth	*\\
		command	/usr/local/bin/HDLspot_repair.pl --start 1>/dev/null 2>/dev/null
EOF
			sed -f $TMP_SCRIPT $CONFIG_CRONTAB > $TMP_CRONTAB 2> /dev/null
			mv $TMP_CRONTAB $CONFIG_CRONTAB
			chmod 666 $CONFIG_CRONTAB
			rm -f $TMP_SCRIPT > /dev/null 2> /dev/null
		fi
	fi

	# raiderror_watchを有効にする
	# sed用のテンポラリファイルを一時的に作成し、これをsedに渡す。
	CONFIG_CRONTAB=$CONFDIR/config-crontab
	if [ -e $CONFIG_CRONTAB ]; then
		if ! grep -q 'raid_errorwatch' $CONFIG_CRONTAB; then
			TMP_SCRIPT=/tmp/tmp.config-crontab.script
			TMP_CRONTAB=/tmp/config-crontab

			rm -f $TMP_SCRIPT > /dev/null 2> /dev/null
			cat <<EOF > $TMP_SCRIPT
/^crontab/ a \\
	raid_errorwatch\\
		hour	*\\
		minute	57\\
		month	*\\
		user	root\\
		dayofweek	*\\
		dayofmonth	*\\
		command	/usr/local/bin/raiderror_watch >/dev/null 2>/dev/null
EOF
			sed -f $TMP_SCRIPT $CONFIG_CRONTAB > $TMP_CRONTAB 2> /dev/null
			mv $TMP_CRONTAB $CONFIG_CRONTAB
			chmod 666 $CONFIG_CRONTAB
			rm -f $TMP_SCRIPT > /dev/null 2> /dev/null
		fi
	fi


	# config-alertに"repeat"エントリを追加
	if ! grep -q 'repeat' $CONFDIR/config-alert; then
		cat <<EOF >> $CONFDIR/config-alert
repeat
	space
		enable	0
	ups
		enable	0
	raid
		enable	0
EOF
	fi


	# 1.21 ftp bug fix
	sed 's/local_umask\t022/local_umask\t002/' $CONFDIR/config-ftp > /tmp/.config-ftp 2>/dev/null || return 1
	mv /tmp/.config-ftp ${CONFDIR}/config-ftp || return 1
	chown 0:0 ${CONFDIR}/config-ftp
	chmod 666 ${CONFDIR}/config-ftp
	sed 's/local_umask=022/local_umask=002/' /etc/vsftpd.conf > /tmp/.vsftpd.conf 2>/dev/null || return 1
	mv /tmp/.vsftpd.conf /etc/vsftpd.conf || return 1
	chown 0:0 /etc/vsftpd.conf
	chmod 644 /etc/vsftpd.conf

	return 0
}

###############################################################################
# crontabの更新
###############################################################################
__update_crontab() {
	CRONTAB_FILE=/etc/crontab

	# アクティブリペアを有効にする

	if [ -e $CRONTAB_FILE ]; then
		if ! grep -q 'HDLspot_repair' $CRONTAB_FILE; then
			cat <<EOF >> $CRONTAB_FILE
0 2   * * 0   root   /usr/local/bin/HDLspot_repair.pl --start 1>/dev/null 2>/dev/null
EOF
		fi
	fi

	# raiderror_watchを有効にする

	if [ -e $CRONTAB_FILE ]; then
		if ! grep -q 'raiderror_watch' $CRONTAB_FILE; then
			cat <<EOF >> $CRONTAB_FILE
57 *   * * *   root   /usr/local/bin/raiderror_watch >/dev/null 2>/dev/null
EOF
		fi
	fi


	return 0
}

###############################################################################
# 1.12のデフォルト設定の不備を修正する
###############################################################################
__fix_112_bug() {
    local md5tgz_sum="38ae97b40da1326a6d786791b60b996c"
    local SUM=""
    local MD5TGZ='/boot/.landisk/md5.tgz'

    # md5.tgz
    if [ ! -e ${MD5TGZ} ]; then
	return 1
    fi
    # 設定に不備のあるmd5.tgzか？
    SUM=`md5sum "${MD5TGZ}" | awk '{print $1}'`
    if [ "x${md5tgz_sum}" != "x${SUM}" ]; then
	return 0
    fi
    # 上書き修正	
    tar xpzf ${FIXFILE} -C / || return 1

    return 0
}

###############################################################################
# ディスクは HDL-GT のシステムを含むか
###############################################################################
__is_system_disk() {
    local PORT=${1}
    local DEV=${2}
    local RESULT=1
    local MAGIC="$(__disklabel_ddread ${DEV} 512 4)"
    local USAGE="$(__disklabel_ddread ${DEV} 520 4)"
    local MODEL="$(__disklabel_ddread ${DEV} 528 16)"
    local NUM_OF_PART=0

    [ "x${MAGIC}" != "xHDLG" ] && return ${RESULT}
    # パーティションが正常か
    sfdisk -V -q "/dev/${DEV}" || return ${RESULT}
    # RAID or EXRD ?
    case "${PORT}" in
	sata1|sata2|sata3|sata4)
	    [ "x${USAGE}" != "xRAID" ] && return ${RESULT}
	    ;;
	esata1)
	    [ "x${USAGE}" != "xEXRD" ] && return ${RESULT}
	    ;;
	*)
	    return ${RESULT}
	    ;;
    esac
    # モデル名が正しいか
    [ $(echo ${MODEL} | grep -c "HDL-GT") -ne 1 ] && return ${RESULT}

    # パーティションが 6 個あるか
    NUM_OF_PART=$(sfdisk -l "/dev/${DEV}" | grep -c "^/dev/${DEV}")
    if [ ${NUM_OF_PART} -ne 6 ]; then
	return ${RESULT}
    fi
    return 0
}


###############################################################################
# disklabel_ident を追加する
###############################################################################
__edit_ident_entry() {
    local IDENT="${1}"
    local HEAD="${2}"
    local DISTFILE="/etc/landisk/conffiles/config-hotplug"
    local TMPFILE="/tmp/.config-hotplug"
    local RESULT=1

    [ "x${IDENT}" = "x" -o "x${HEAD}" = "x" ] && return ${RESULT}

    # 置換
    sed -e '/^'"${HEAD}"'/,/^[a-zA-Z0-9]/{/disklabel_ident/D}' -e '/^'"${HEAD}"'/a\	disklabel_ident	'"${IDENT}" ${DISTFILE} > ${TMPFILE}
    if [ ${?} -ne 0 ]; then
	return ${RESULT}
    fi

    mv ${TMPFILE} ${DISTFILE}
    chown 0:0 ${DISTFILE}
    chmod 666 ${DISTFILE}

    return 0
}

###############################################################################
# disklabel_identフィールドの生成
###############################################################################
__make_ident() {
    local DISK=${1}
    local RESULT=1
    local SUM=""

    if [ "x${DISK}" = "x" ]; then
	return ${RESULT}
    fi

    SUM=`/usr/local/bin/mv_identify -b /dev/${DISK} | md5sum | awk '{print $1}'`
    if [ "x${SUM}" = "x" ]; then
	return ${RESULT}
    elif [ "${SUM}" = "d41d8cd98f00b204e9800998ecf8427e" ]; then
	# fail dd (blank)
	return ${RESULT}
    fi
    echo "${SUM}"
    return 0
}

###############################################################################
# ディスクラベルの各項目を書き込む
###############################################################################
__ddwrite() {
     local DEVICE="${1}"
     local OFFSET="${2}"
     local LENGTH="${3}"
     local DATA="${4}"
     local ZEROS="$(dd if=/dev/zero bs=1 count=${LENGTH} 2>/dev/null)"

     # まずこれから書き込む領域を 0 クリアする
     /bin/dd \
	 if=/dev/zero \
	 of="/dev/${DEVICE}" \
	 bs=1 \
	 seek=${OFFSET} \
	 count=${LENGTH} \
	 1>/dev/null 2>/dev/null

     # データの書き込み
     ( echo -n "${DATA}${ZEROS}" | \
	 /bin/dd \
	 of="/dev/${DEVICE}" \
	 bs=1 \
	 seek=${OFFSET} \
	 count=${LENGTH} \
	 1>/dev/null 2>/dev/null ) || return 1

     return 0
}

###############################################################################
# RAID モードに応じてディスクの接続状態を確認する
#   入力例: sda
#   出力例: sata1
#############################################################################
__portname_to_dev() {
    local PORT=${1}
    local RESULT=1
    local IDENTIFER=""
    local DEV=""
    local i=0

    case "${PORT}" in
	esata1)
	    IDENTIFER='devices/platform/host0'
	    ;;
	esata2)
	    IDENTIFER='devices/platform/host1'
	    ;;
	sata1)
	    IDENTIFER='devices/pci.*host2'
	    ;;
	sata2)
	    IDENTIFER='devices/pci.*host3'
	    ;;
	sata3)
	    IDENTIFER='devices/pci.*host4'
	    ;;
	sata4)
	    IDENTIFER='devices/pci.*host5'
	    ;;
	usb1)
	    IDENTIFER='usb1/1-1'
	    ;;
	usb2)
	    IDENTIFER='usb2/2-1'
	    ;;
	*)
	    return ${RESULT}
	    ;;
    esac

    [ ! -d /sys/block ] && return ${RESULT}
    for DEV in `(cd /sys/block ; ls -d sd?)`
    do
	if readlink /sys/block/${DEV}/device | grep -q ${IDENTIFER}; then
	    echo "${DEV}"
	    RESULT=0
	    break
	fi
    done
    return ${RESULT}
}

###############################################################################
# disklabel_identの追加
###############################################################################
__add_disklabel_ident() {
    local PORT=""
    local DEV=""
    local IDENT=""
    local RESULT=1

    # 各ポートが存在している時のみ処理
    for PORT in sata1 sata2 sata3 sata4 esata1
    do
    	# sata1 --> sda 変換
	DEV=$(__portname_to_dev ${PORT})
	[ "x${DEV}" = "x" ] && continue
	__is_system_disk "${PORT}" "${DEV}" || continue
	IDENT=$(__make_ident ${DEV}) || return ${RESULT}
	# ident(rev.0002)
	__ddwrite "${DEV}" 592 32 "${IDENT}" || return ${RESULT}
	# config-hotplug
	__edit_ident_entry "${IDENT}" "${PORT}" || return ${RESULT}
    done
    return 0
}

###############################################################################
# reset script の書換
###############################################################################
__edit_reset_script() {
    local NORMAL='/usr/local/share/reinitialize/reset_normal.sh'
    local FULL='/usr/local/share/reinitialize/reset_full.sh'
    local TMPFILE='/tmp/.__edit_reset_script'
    local f=""

    for f in ${NORMAL} ${FULL}
    do
	# HDL-GTR --> HDL-GT へ置換
	sed -e 's/^MODEL="HDL-GTR%s"/MODEL="HDL-GT%s"/' \
	    -e 's/^USAGE="HDL-GTR"/USAGE="HDL-GT"/' \
	    ${f} > ${TMPFILE} || return 1
	mv ${TMPFILE} ${f} || return 1
	chown 0:0 ${f}
	chmod 755 ${f}
    done
}

###############################################################################
# MACアドレスを10進数に変換する
###############################################################################
__mac_addr_to_dec() {
	local hexaddr="$1"

	if [ "x$hexaddr" = 'x' ]; then
		return 1
	fi

	hexaddr=`echo $hexaddr | sed 's/://g'`
	printf "%d" 0x$hexaddr
}

###############################################################################
# 書き換え範囲かチェックをする
# return 0:対象 1:非対象
###############################################################################
__can_r8_update() {
	local thaddr="$THADDR"
	local thaddr_r2u="$THADDR_R2U"
	local ownaddr=$(LANG=C /sbin/ifconfig eth0 | sed -n 's/.* HWaddr \([0-9A-Fa-f:]*\)/\1/p')

	if [ "x$ownaddr" = 'x' ]; then
		return 1
	fi

	# HDL-GTR2Uは変更する
	if `cat /proc/cmdline | grep -q model=HDL-GTR2U >&/dev/null`; then
		thaddr="$thaddr_r2u"
	fi

	local own=$(__mac_addr_to_dec $ownaddr)
	local threshold=$(__mac_addr_to_dec $thaddr)

	[ "x$own" = 'x' ] && return 1
	[ "x$threshold" = 'x' ] && return 1

	if [ $own -lt $threshold ]; then
		return 0
	fi
	return 1
}


########################
# start
start_led


# perlでアップデート
${UPPERL} --update
if [ $? != 0 ]; then
    exit_select 3
fi

# サービスの停止の布石
SERVICE=`${UPPERL} --getservice 2>/dev/null | awk -F '=' '{print $1}'`

# remount rw /
mount -o remount,rw,noatime /

# サービスの停止
for i in $SERVICE apache2 HDLFind.sh cron xinetd sysklogd networking
do
    /etc/init.d/$i stop
done

# 必要であれば
#killall buttond
#killall usbhdmng

# 1.12のための処理
if [ "x${VERSION}" = "x1.12" -a "x${MODEL}" = "xHDL-GT" ]; then
    __fix_112_bug || exit_select 6
fi

# tarで展開
tar xpzf ${TARFILE} -C / || exit_select 1

# rmlistによる削除
if [ -e ${RMLIST} ]; then
    cat ${RMLIST} | xargs rm -f
fi

# conffilesのアップデート
__update_conffiles || exit_select 1

# crontabのアップデート
__update_crontab || exit_select 1

# disklabel_identの追加
__add_disklabel_ident || exit_select 1

# reset scriptの書換 (GTのみ)
if [ "x${MODEL}" = "xHDL-GT" ]; then
    __edit_reset_script || exit_select 1
fi

# /mnt/rom0の作成
if [ ! -d /mnt/rom0 ]; then
    mkdir /mnt/rom0 || exit_select 1
fi

# R8の更新
if ! pidof R8contd >/dev/null 2>&1; then
    /usr/local/sbin/R8contd
fi
if __can_r8_update; then
	/usr/local/bin/R8fwrite -p iodata -d $MOTDEF2 || exit_select 4
else
	/usr/local/bin/R8fwrite -p iodata -d $MOTDEF || exit_select 4
fi

# ファイルを削除する
rm -f $TARFILE $BETAFILE $VERFILE $CONFTAR $RMLIST $FIXFILE $MOTDEF $MOTFILE $MOTDEF2 $MOTFILE2

# zoneinfo設定
/usr/local/bin/HDLsystem --set || exit_select 1

# 終了処理
upopts=--setversion=$UPDATEVER
[ "$UPDATESUBVER" != "" ] && upopts="$upopts --setsubversion=$UPDATESUBVER"
[ "$BETA" != "" ] && upopts="$upopts --setbeta=$BETA"

echo ${UPPERL} $upopts >/dev/console
${UPPERL} $upopts
rm -f $UPPERL
echo "success($UPDATEVER)" > /mnt/hda5/.update/update.log
